@@ -5,9 +5,8 @@ import pandas as pd
 import lxml.etree as etree
 from lxml.etree import Element, SubElement, parse
 from datetime import datetime
-import importlib
-import os.path
 import sys
+import chardet
 if sys.version_info < (3, 9):
     # importlib.resources either doesn't exist or lacks the files()
     # function, so use the PyPI version:
@@ -15,29 +14,29 @@ if sys.version_info < (3, 9):
 else:
     # importlib.resources has files(), so use that:
     import importlib.resources as importlib_resources
+import os.path
 
 ###############################################################################
-## Please, edit this part if necessary
+# Please, edit this part if necessary
 editor = 'Fernando Sanz-Lázaro'
 authority = 'University of Vienna, Institute of Romance Languages '\
     'and Literatures'
-publisher_id = 'dracor'
-publisher_name = 'DraCor'
+publisher_id = ''
+publisher_name = ''
 licence = 'CC BY 3.0'
 licence_url = 'https://creativecommons.org/publicdomain/by/3.0/'
 hodie = datetime.today().strftime('%Y-%m-%d')
 nunc = datetime.today().strftime('%H:%M:%S')
-editor = ''
 authority = ''
-publisher = ('DraCor', 'dracor', 'https://dracor.org')
+publisher = ('', '', '')
 licence = 'CC BY'
 licence_url = 'https://creativecommons.org/publicdomain/BY/3.0/'
 xml_model = '<?xml-model href="https://dracor.org/schema.rng" type="applicati'\
         'on/xml" schematypens="http://relaxng.org/ns/structure/1.0"?>'
-edition = '0.1'
 ###############################################################################
 
-def make_tree(title, subtitle, author, source, date,
+
+def make_tree(title, subtitle, author, genre, subgenre, source, date,
               authority, publisher, licence, speakers_list):
     root = Element('TEI',
                    xmlns='http://www.tei-c.org/ns/1.0')
@@ -78,14 +77,16 @@ def make_tree(title, subtitle, author, source, date,
     tei_header.append(make_revision())
     # Construct text and body
     standoff = SubElement(root, 'standOff')
-    standoff = make_standoff(standoff)
+    standoff = make_standoff(standoff, date)
 
     SubElement(root, 'text')
     return tree
 
 
 def parse_date(date):
-    if '-' in date:
+    if not date or len(date) == 0:
+        date = {'notbefore': '1550', 'notafter': '1699'}
+    elif '-' in date:
         date = {'notbefore': date.split('-')[0],
                 'notafter': date.split('-')[1]}
     elif '?' in date:
@@ -152,7 +153,7 @@ def make_source(source_desc, source):
     return source_desc
 
 
-def make_standoff(stand_off):
+def make_standoff(stand_off, date):
     list_event = SubElement(stand_off, 'listEvent')
     SubElement(list_event, 'date', date, type='print')
     SubElement(list_event, 'date', date, type='written')
@@ -215,7 +216,7 @@ def make_id(name):
 def find_characters(play_file):
     try:
         nombresdf = pd.read_csv('sexos.csv')
-    except:
+    except Exception:
         nombresdf = pd.DataFrame(columns=['Personaje', 'Sexo', 'Comprobado'])
     names = []
     characters_dict = {}
@@ -339,7 +340,7 @@ def parse_pronoun(pronoun, on_stage, characters_list):
     else:
         sex = 'FEMALE'
     return [[person for person in on_stage
-             if characters_list[person][1] == sex][-1]]
+             if characters_list[person][1] == sex]][-1]
 
 
 def parse_exit(on_stage, line):
@@ -357,11 +358,11 @@ def parse_exit(on_stage, line):
     return on_stage
 
 
-def parse_speech(ln, nextl, nl):
+def parse_speech(ln, nextl, nl, p=False):
     tabs = ln.count('\t')
     tabsn = nextl.count('\t')
     text = re.search('(?:<.*>)*(.*)', ln.strip()).group(1)
-    if '<p>' in text:
+    if p:
         line = Element('p')
     else:
         line = Element('l')
@@ -370,10 +371,11 @@ def parse_speech(ln, nextl, nl):
                 line.set('part', 'M')
             else:
                 line.set('part', 'I')
+            line.set('n', f'{nl + 1}')
         else:
             nl += 1
-            if nl % 5 == 0:
-                line.set('n', f'{nl}')
+            # if nl % 5 == 0:
+            line.set('n', f'{nl}')
             if tabs > 1:
                 line.set('part', 'F')
     line.text = text
@@ -384,6 +386,28 @@ def compare_sexes(pids, characters_list):
     return 'UNKNOWN'
 
 
+def parse_author(name):
+    if os.path.isfile('authors.xml'):
+        fauthors = parse('authors.xml')
+    else:
+        with importlib_resources.path('txt2tei', 'authors.xml') as dat:
+            fauthors = parse(dat)
+    names = name.split()
+    if len(names) > 1:
+        cert = 'medium'
+    else:
+        cert = 'high'
+    authors = [a for a in fauthors.xpath('author')
+               if any(b.text == names[0] for b in a.xpath('persName/*'))]
+    if authors:
+        author = [a for a in authors if a.get('cert') == cert][0]
+    else:
+        author = Element('Author')
+        b = SubElement(author, 'persName')
+        b.text = name
+    return author
+
+
 def parse_name(ln, characters_list, on_stage):
     if '#' in ln:
         splitted = ln.split('#')
@@ -426,13 +450,14 @@ def parse_name(ln, characters_list, on_stage):
 header_labels = {'t': 'title', 'tt': 'subtitle', 'a': 'author',
                  'g': 'genre', 's': 'subgenre', 'o': 'source',
                  'f': 'date', 'x': 'comment'}
+rev_header_labels = dict((v, k) for k, v in header_labels.items())
 body_labels = {'j': 'act', '<e>': 'echo', '<p>': 'prose',
                '<i>': 'stage_direction', '<x>': 'comment'}
 subtitle = ''
 author = ''
 genre = ''
 subgenre = ''
-date = ''
+# date = ''
 source = ''
 on_stage = []
 dramatis_personae = Element('null')
@@ -445,60 +470,47 @@ reference = {'1º': 'I', '1.º': 'I', '2º': 'II', '2.º': 'II', '3º': 'III',
              'SEGUNDA': 'ii', 'TERCERA': 'iii', 'CUARTA': 'iv'}
 
 
+def unixfy(filename):
+    with open(filename, 'rb') as file:
+        raw = file.read(32)
+        encoding = chardet.detect(raw)['encoding']
+    return encoding
+
+
 def main(input_arguments=sys.argv):
     input_file = input_arguments[1]
     output = f'{input_file.rsplit(".", 1)[0]}.xml'
-    if len(input_arguments) > 2:
-        edition = input_arguments[2]
-
-    with open(input_file) as f:
+    enc = unixfy(input_file)
+    with open(input_file, 'rU', encoding=enc) as f:
         characters_list = find_characters(f)
-    with open(input_file) as g:
-        lines = g.readlines()
-        if os.path.isfile('authors.xml'):
-            fauthors = parse('authors.xml')
-        else:
-            with importlib.resources.path('txt2tei', 'authors.xml') as dat:
-                fauthors = parse(dat)
-       # fauthors = parse('authors.xml')
+        f.seek(0)
+        lines = f.readlines()
+    author = date = title = n = sp = ''
     for line in lines:
-        global author
-        global date
-        global title
-        global n
-        global sp
         if line.startswith('<'):
             label = re.search(r'^\s*<(\w*)>', line.strip()).group(1)
             if label in header_labels.keys():
-                value = line.strip(f'<{label}>').strip()
-                variable = f'{header_labels[label]}'
-                globals()[variable] = value
+                header_labels[label] = line.strip(f'<{label}>').strip()
             else:
                 break
         else:
             break
-        if date:
-            date = parse_date(date)
-    author = author.split()
-    if not date:
+    if header_labels['f']:
+        date = parse_date(header_labels['f'])
+    else:
         date = parse_date('')
-    if len(author) > 1:
-        cert = 'medium'
-    else:
-        cert = 'high'
-    author = author[0]
-    authors = [a for a in fauthors.xpath('author')
-               if any(b.text == author for b in a.xpath('persName/*'))]
-    if authors:
-        author = [a for a in authors if a.get('cert') == cert][0]
-    else:
-        a = Element('persName')
-        a.text = author
-        author = a
+    author = parse_author(header_labels['a'])
+    title = header_labels['t']
+    subtitle = header_labels['tt']
+    genre = header_labels['g']
+    subgenre = header_labels['s']
+    source = header_labels['o']
     tree = make_tree(
         title,
         subtitle,
         author,
+        genre,
+        subgenre,
         source,
         date,
         authority,
@@ -508,7 +520,6 @@ def main(input_arguments=sys.argv):
     text = tree.xpath('/TEI/text')[0]
     front = SubElement(text, 'front')
     body = SubElement(text, 'body')
-    dict_personas = {}
     count = 0
     actn = 0
     scenen = 1
@@ -520,10 +531,11 @@ def main(input_arguments=sys.argv):
         next_line = ''
         if not any(line.startswith(x) for x in header_labels):
             if line.strip() in reference.keys():
-                print(f'{input_file}:\tPlease, edit manually <sp who="#character">'
-                      f'for {line.strip()} in {output} verse {n}\n')
+                print(f'{input_file}:\tPlease, edit manually'
+                      '<sp who="#character"> for '
+                      f'{line.strip()} in {output} verse {n}\n')
             if line.startswith('<el>'):
-                dramatis_personae = parse_cast(value)
+                dramatis_personae = parse_cast(line)
                 front.append(dramatis_personae)
             elif line.startswith('<j>') or line.startswith('<q>'):
                 on_stage = []
@@ -533,13 +545,15 @@ def main(input_arguments=sys.argv):
                     scenen = 1
                     if actn > 1:
                         act = SubElement(body, 'div', type='act', n=f'{actn}')
-                        scene = SubElement(act, 'div', type='scene', n=f'{scenen}')
+                        scene = SubElement(act, 'div', type='scene',
+                                           n=f'{scenen}')
                     if head and len(head) > 1:
                         SubElement(act, 'head').text = head
                 else:
                     scenen += 1
                     if scenen > 1:
-                        scene = SubElement(act, 'div', type='scene', n=f'{scenen}')
+                        scene = SubElement(act, 'div', type='scene',
+                                           n=f'{scenen}')
             elif line.strip().startswith('<i>'):
                 text = f'{line.replace("<i>","").strip()}'
                 if line.startswith('\t'):
@@ -554,6 +568,10 @@ def main(input_arguments=sys.argv):
                 scene.append(sp)
                 on_stage = parsed_line[1]
             elif line.startswith('\t') or line.strip().startswith('<p>'):
+                if line.strip().startswith('<p>'):
+                    p = True
+                else:
+                    p = False
                 for i in list(range(1, 9)):
                     if idx+i < len(lines):
                         if any([x in lines[idx+i] for x in ['<i>', '<x>']]):
@@ -565,10 +583,11 @@ def main(input_arguments=sys.argv):
                             pass
                     else:
                         next_line = '\tFinal'
-                parsed_line = parse_speech(line, next_line, count)
+                parsed_line = parse_speech(line, next_line, count, p)
                 code = parsed_line[0]
-                n = parsed_line[1]
+                count = parsed_line[1]
                 sp.append(code)
     tree.write(output, doctype=xml_model, encoding='UTF-8', pretty_print=True)
 
+
 main()
